@@ -175,6 +175,82 @@ static int dt_set_rng_seed_adt(int node)
     return 0;
 }
 
+static int dt_set_fb(void)
+{
+    if (!cur_boot_args.video.base) {
+        printf("FDT: Framebuffer unavailable, skipping framebuffer initialization");
+        return 0;
+    }
+
+    int fb = fdt_path_offset(dt, "/chosen/framebuffer");
+
+    if (fb < 0) {
+        printf("FDT: No framebuffer found\n");
+        return 0;
+    }
+
+    u64 fb_base, fb_height;
+    get_notchless_fb(&fb_base, &fb_height);
+    u64 fb_size = cur_boot_args.video.stride * fb_height;
+    u64 fbreg[2] = {cpu_to_fdt64(fb_base), cpu_to_fdt64(fb_size)};
+    char fbname[32];
+
+    snprintf(fbname, sizeof(fbname), "framebuffer@%lx", fb_base);
+
+    if (fdt_setprop(dt, fb, "reg", fbreg, sizeof(fbreg)))
+        bail("FDT: couldn't set framebuffer.reg property\n");
+
+    if (fdt_set_name(dt, fb, fbname))
+        bail("FDT: couldn't set framebuffer name\n");
+
+    if (fdt_setprop_u32(dt, fb, "width", cur_boot_args.video.width))
+        bail("FDT: couldn't set framebuffer width\n");
+
+    if (fdt_setprop_u32(dt, fb, "height", fb_height))
+        bail("FDT: couldn't set framebuffer height\n");
+
+    if (fdt_setprop_u32(dt, fb, "stride", cur_boot_args.video.stride))
+        bail("FDT: couldn't set framebuffer stride\n");
+
+    const char *format = NULL;
+
+    switch (cur_boot_args.video.depth & 0xff) {
+        case 32:
+            format = "x8r8g8b8";
+            break;
+        case 30:
+            format = "x2r10g10b10";
+            break;
+        case 16:
+            format = "r5g6b5";
+            break;
+        default:
+            printf("FDT: unsupported fb depth %lu, not enabling\n", cur_boot_args.video.depth);
+            return 0; // Do not error out, but don't set the FB
+    }
+
+    if (fdt_setprop_string(dt, fb, "format", format))
+        bail("FDT: couldn't set framebuffer format\n");
+
+    fdt_delprop(dt, fb, "status"); // may fail if it does not exist
+
+    printf("FDT: %s base 0x%lx size 0x%lx\n", fbname, fb_base, fb_size);
+
+    // We do not need to reserve the framebuffer, as it will be excluded from the usable RAM
+    // range already.
+
+    // save notch height in the dcp node if present
+    if (cur_boot_args.video.height - fb_height) {
+        int dcp = fdt_path_offset(dt, "dcp");
+        if (dcp >= 0)
+            if (fdt_appendprop_u32(dt, dcp, "apple,notch-height",
+                                   cur_boot_args.video.height - fb_height))
+                printf("FDT: couldn't set apple,notch-height\n");
+    }
+
+    return 0;
+}
+
 static int dt_set_chosen(void)
 {
 
@@ -207,70 +283,9 @@ static int dt_set_chosen(void)
         printf("FDT: initrd at %p size 0x%lx\n", initrd_start, initrd_size);
     }
 
-    if (cur_boot_args.video.base) {
-        int fb = fdt_path_offset(dt, "/chosen/framebuffer");
-        if (fb < 0)
-            bail("FDT: /chosen node not found in devtree\n");
+    if (dt_set_fb())
+        return -1;
 
-        u64 fb_base, fb_height;
-        get_notchless_fb(&fb_base, &fb_height);
-        u64 fb_size = cur_boot_args.video.stride * fb_height;
-        u64 fbreg[2] = {cpu_to_fdt64(fb_base), cpu_to_fdt64(fb_size)};
-        char fbname[32];
-
-        snprintf(fbname, sizeof(fbname), "framebuffer@%lx", fb_base);
-
-        if (fdt_setprop(dt, fb, "reg", fbreg, sizeof(fbreg)))
-            bail("FDT: couldn't set framebuffer.reg property\n");
-
-        if (fdt_set_name(dt, fb, fbname))
-            bail("FDT: couldn't set framebuffer name\n");
-
-        if (fdt_setprop_u32(dt, fb, "width", cur_boot_args.video.width))
-            bail("FDT: couldn't set framebuffer width\n");
-
-        if (fdt_setprop_u32(dt, fb, "height", fb_height))
-            bail("FDT: couldn't set framebuffer height\n");
-
-        if (fdt_setprop_u32(dt, fb, "stride", cur_boot_args.video.stride))
-            bail("FDT: couldn't set framebuffer stride\n");
-
-        const char *format = NULL;
-
-        switch (cur_boot_args.video.depth & 0xff) {
-            case 32:
-                format = "x8r8g8b8";
-                break;
-            case 30:
-                format = "x2r10g10b10";
-                break;
-            case 16:
-                format = "r5g6b5";
-                break;
-            default:
-                printf("FDT: unsupported fb depth %lu, not enabling\n", cur_boot_args.video.depth);
-                return 0; // Do not error out, but don't set the FB
-        }
-
-        if (fdt_setprop_string(dt, fb, "format", format))
-            bail("FDT: couldn't set framebuffer format\n");
-
-        fdt_delprop(dt, fb, "status"); // may fail if it does not exist
-
-        printf("FDT: %s base 0x%lx size 0x%lx\n", fbname, fb_base, fb_size);
-
-        // We do not need to reserve the framebuffer, as it will be excluded from the usable RAM
-        // range already.
-
-        // save notch height in the dcp node if present
-        if (cur_boot_args.video.height - fb_height) {
-            int dcp = fdt_path_offset(dt, "dcp");
-            if (dcp >= 0)
-                if (fdt_appendprop_u32(dt, dcp, "apple,notch-height",
-                                       cur_boot_args.video.height - fb_height))
-                    printf("FDT: couldn't set apple,notch-height\n");
-        }
-    }
     node = fdt_path_offset(dt, "/chosen");
     if (node < 0)
         bail("FDT: /chosen node not found in devtree\n");
@@ -378,7 +393,7 @@ static int dt_set_memory(void)
                 continue;
             }
 
-            if ((resv_start | resv_len) & (SZ_16K - 1)) {
+            if ((resv_start | resv_len) & (get_page_size() - 1)) {
                 bail("FDT: reserved-memory node %s not page-aligned, ignoring (%lx..%lx)\n", name,
                      resv_start, resv_end);
                 continue;
@@ -426,6 +441,73 @@ static int dt_set_serial_number(void)
     return 0;
 }
 
+static int dt_set_smbios(void)
+{
+    int fdt_root = fdt_path_offset(dt, "/");
+    int adt_root = adt_path_offset(adt, "/");
+    int len, node, smbios_node;
+
+    if (fdt_root < 0)
+        bail("FDT: could not open a handle to FDT root.\n");
+    if (adt_root < 0)
+        bail("ADT: could not open a handle to ADT root.\n");
+
+    node = fdt_add_subnode(dt, fdt_root, "smbios");
+    if (node < 0)
+        bail("FDT: failed to add node 'smbios' to  '/'\n");
+    fdt_setprop_string(dt, node, "compatible", "u-boot,sysinfo-smbios");
+
+    smbios_node = fdt_add_subnode(dt, node, "smbios");
+    if (smbios_node < 0)
+        bail("FDT: failed to add node 'smbios' to  '/smbios'\n");
+
+    node = fdt_add_subnode(dt, smbios_node, "system");
+    if (node < 0)
+        bail("FDT: failed to add node 'system' to  '/smbios'\n");
+
+    const char *manuf = adt_getprop(adt, adt_root, "manufacturer", NULL);
+    if (!manuf)
+        bail("ADT: failed to get manufacturer property\n");
+    fdt_setprop_string(dt, node, "manufacturer", manuf);
+    const char *model = adt_getprop(adt, adt_root, "model", NULL);
+    if (!model)
+        bail("ADT: failed to get model property\n");
+    fdt_setprop_string(dt, node, "product", model);
+    const char *serial = adt_getprop(adt, adt_root, "serial-number", NULL);
+    if (!serial)
+        bail("ADT: failed to get serial-number property\n");
+    fdt_setprop_string(dt, node, "serial", serial);
+
+    node = fdt_add_subnode(dt, smbios_node, "baseboard");
+    if (node < 0)
+        bail("FDT: failed to add node 'baseboard' to  '/smbios'\n");
+    fdt_setprop_string(dt, node, "manufacturer", manuf);
+    const char *target_type = adt_getprop(adt, 0, "target-type", NULL);
+    if (!target_type)
+        bail("ADT: failed to get target-type property\n");
+    fdt_setprop_string(dt, node, "product", target_type);
+
+    node = fdt_add_subnode(dt, smbios_node, "chassis");
+    if (node < 0)
+        bail("FDT: failed to add node 'chassis' to  '/smbios'\n");
+    fdt_setprop_string(dt, node, "manufacturer", manuf);
+    u32 enclosure = 0x02;
+    const char *chassis_type = fdt_getprop(dt, fdt_root, "chassis-type", &len);
+    if (chassis_type) {
+        if (!strcmp(chassis_type, "desktop"))
+            enclosure = 0x03;
+        else if (!strcmp(chassis_type, "laptop"))
+            enclosure = 0x09;
+        else if (!strcmp(chassis_type, "all-in-one"))
+            enclosure = 0x0d;
+        else if (!strcmp(chassis_type, "server"))
+            enclosure = 0x11;
+    }
+    fdt_setprop_u32(dt, node, "chassis-type", enclosure);
+
+    return 0;
+}
+
 static int dt_set_cpus(void)
 {
     int ret = 0;
@@ -653,6 +735,10 @@ static struct {
         .alias = "ethernet0",
         .fdt_property = "local-mac-address",
     },
+    {
+        .alias = "ethernet1",
+        .fdt_property = "local-mac-address",
+    },
     {
         .alias = "wifi0",
         .fdt_property = "local-mac-address",
@@ -701,8 +787,10 @@ static int dt_set_bluetooth_cal(int anode, int node, const char *adt_name, const
     u32 len;
     const u8 *cal_blob = adt_getprop(adt, anode, adt_name, &len);
 
-    if (!cal_blob || !len)
-        bail("ADT: Failed to get %s\n", adt_name);
+    if (!cal_blob || !len) {
+        printf("ADT: Failed to get %s. Replaced module?\n", adt_name);
+        return 0;
+    }
 
     fdt_setprop(dt, node, fdt_name, cal_blob, len);
     return 0;
@@ -905,8 +993,10 @@ static int dt_set_wifi(void)
     u32 len;
     const u8 *cal_blob = adt_getprop(adt, anode, "wifi-calibration-msf", &len);
 
-    if (!cal_blob || !len)
-        bail("ADT: Failed to get wifi-calibration-msf\n");
+    if (!cal_blob || !len) {
+        printf("ADT: Failed to get wifi-calibration-msf. Replaced module?\n");
+        return 0;
+    }
 
     fdt_setprop(dt, node, "brcm,cal-blob", cal_blob, len);
 
@@ -1818,7 +1908,7 @@ static int dt_set_display(void)
     dart_lock_adt("/arm-io/dart-disp0", 0);
 
     /* Add "/reserved-memory" nodes with iommu mapping and link them to their
-     * devices. The memory is already excluded from useable RAM so these nodes
+     * devices. The memory is already excluded from usable RAM so these nodes
      * are only required to inform the OS about the existing mappings.
      * Required for disp0, dcp and all dcpext.
      * Checks for dcp* / disp*_piodma / disp* aliases and fails silently if
@@ -2383,6 +2473,43 @@ static int dt_transfer_virtios(void)
     return 0;
 }
 
+/*
+ * Remove unneeded apple,always-on from unused memory channels on T8012.
+ * This is needed because the amount of memory and the amount of used
+ * memory channels depends on the storage configuration of the host Mac
+ * and can only be determined at runtime.
+ */
+static int dt_set_pmgr(void)
+{
+    if (chip_id != T8012)
+        return 0;
+
+    if (mem_size_actual > 0x40000000)
+        return 0;
+
+    int pmgr_node = fdt_path_offset(dt, "/soc/power-management@20e000000");
+    if (pmgr_node < 0) {
+        printf("FDT: Failed to find pmgr node\n");
+        return 0;
+    }
+
+    int dcs2_node = fdt_path_offset(dt, "/soc/power-management@20e000000/power-controller@80258");
+    if (dcs2_node < 0)
+        bail("FDT: failed to find ps_dcs2 node\n");
+
+    /* Allow failure */
+    fdt_delprop(dt, dcs2_node, "apple,always-on");
+
+    int dcs3_node = fdt_path_offset(dt, "/soc/power-management@20e000000/power-controller@80260");
+    if (dcs3_node < 0)
+        bail("FDT: failed to find ps_dcs3 node\n");
+
+    /* Allow failure */
+    fdt_delprop(dt, dcs3_node, "apple,always-on");
+
+    return 0;
+}
+
 void kboot_set_initrd(void *start, size_t size)
 {
     initrd_start = start;
@@ -2525,13 +2652,15 @@ int kboot_prepare_dt(void *fdt)
     if (fdt_add_mem_rsv(dt, (u64)_base, ((u64)_end) - ((u64)_base)))
         bail("FDT: couldn't add reservation for m1n1\n");
 
-    /* setup console log buffer early to cpature as much log as possible */
+    /* setup console log buffer early to capture as much log as possible */
     dt_setup_mtd_phram();
 
     if (dt_set_chosen())
         return -1;
     if (dt_set_serial_number())
         return -1;
+    if (dt_set_smbios())
+        return -1;
     if (dt_set_cpus())
         return -1;
     if (dt_set_mac_addresses())
@@ -2570,6 +2699,8 @@ int kboot_prepare_dt(void *fdt)
         return -1;
     if (dt_set_isp_fwdata())
         return -1;
+    if (dt_set_pmgr())
+        return -1;
 #ifndef RELEASE
     if (dt_transfer_virtios())
         return 1;
